-- BergmannRoyal
-- Specialization for BergmannRoyal mod
--
-- @author  Tobias F. (John Deere 6930, 1. LS-Modschmiede)
-- @LastEdit 08/12/09
-- Converted to ls2011 Gnom

BergmannRoyal = {};

function BergmannRoyal.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function BergmannRoyal:load(xmlFile)
	self.PU = {};
	self.PU.limitDown = {Utils.degToRad(getXMLFloat(xmlFile, "vehicle.PU#limitDown"))};
	self.PU.limitUp = {Utils.degToRad(getXMLFloat(xmlFile, "vehicle.PU#limitUp"))};
	self.PU.isDown = false;
	self.PU.rotNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.PU#rotNode"));
	local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.PU#rotScale"));
	self.PU.rotScale = {x, y, z}
	
	self.stripes = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.Stripes#index"));
	self.visiblefruitTypes = {}
	local visiblefruitTypes = getXMLString(xmlFile, "vehicle.Stripes#visibleOn");
    if visiblefruitTypes ~= nil then
        local types = Utils.splitString(" ", visiblefruitTypes);
        for k,v in pairs(types) do
            local desc = FruitUtil.fruitTypes[v];
            if desc ~= nil then
                self.visiblefruitTypes[desc.index] = true;
            end;
        end;
    end;
	
    self.BackRollers = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.backRollers.node(%d)", i);
        local s = getXMLString(xmlFile, key.."#rotScale");
        local index = getXMLString(xmlFile, key.."#index");
        if s==nil or index==nil then
            break;
        end;
		local entry = {};
        local node = Utils.indexToObject(self.components, index);
        if node ~= nil then
            entry.node = node;
        end;
		local x,y,z = Utils.getVectorFromString(s);
		entry.rotScale = {x, y, z}
		table.insert(self.BackRollers, entry);
        i = i +1;
    end;
	
	local forageWgnSound = getXMLString(xmlFile, "vehicle.forageWgnSound#file");
    if forageWgnSound ~= nil and forageWgnSound ~= "" then
        forageWgnSound = Utils.getFilename(forageWgnSound, self.baseDirectory);
        self.forageWgnSound = createSample("forageWgnSound");
        self.forageWgnSoundEnabled = false;
        loadSample(self.forageWgnSound, forageWgnSound, false);
        self.forageWgnSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#pitchOffset"), 1);
        self.forageWgnSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#volume"), 1.0);
    end;


	self.fillNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.fillNode#index"));	
    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);
    self.wasToFast = false;
    self.isTurnedOn = false;
end;



function BergmannRoyal:delete()
    if self.forageWgnSound ~= nil then
        delete(self.forageWgnSound);
    end;
end;

function BergmannRoyal:mouseEvent(posX, posY, isDown, isUp, button)
end;

function BergmannRoyal:keyEvent(unicode, sym, modifier, isDown)
end;

function BergmannRoyal:update(dt)
	if self:getIsActiveForInput() then
		if InputBinding.isPressed(InputBinding.DRAWBAR_UP) then
			self.Go.AttacherMoving = true; 
			self.Done.AttacherMoving = true;
		elseif InputBinding.isPressed(InputBinding.DRAWBAR_DOWN) then
			self.Go.AttacherMoving = false; 
			self.Done.AttacherMoving = true;
		else
			self.Done.AttacherMoving = false; 
		end; 
	
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) and self.PU.isDown then
			self.isTurnedOn = not self.isTurnedOn;
		end;
			
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
			self.PU.isDown = not self.PU.isDown;
		end;
	end;
	
	if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN then
		for k, rollers in pairs(self.BackRollers) do	
			rotate(rollers.node, unpack(rollers.rotScale))
		end;	
	end;
	if self.stripes ~= nil and self.visiblefruitTypes ~= nil then
		if self.currentFillType ~= FruitUtil.FRUITTYPE_UNKNOWN then
			if self.visiblefruitTypes[self.currentFillType] then
				setVisibility(self.stripes, true);
			else
				setVisibility(self.stripes, false);
			end;
		else
			setVisibility(self.stripes, true);
		end;
	end;
	
    self.wasToFast = false;
	if self:getIsActive() then
		if self.isTurnedOn  and (self:allowFillType(FruitUtil.FRUITTYPE_GRASS) or self:allowFillType(FruitUtil.FRUITTYPE_DRYGRASS)) and self.capacity > self.fillLevel then
			if self.PU.rotNode ~= nil then
				rotate(self.PU.rotNode, unpack(self.PU.rotScale))
			end;
			local toFast = self:doCheckSpeedLimit() and self.attacherVehicle.lastSpeed*3600 > 29;
			if not toFast and wheel.hasGroundContact then
				for k, cuttingArea in pairs(self.cuttingAreas) do
					local x,y,z = getWorldTranslation(cuttingArea.start);
					local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
					local x2,y2,z2 = getWorldTranslation(cuttingArea.height);

					local area = Utils.updateCuttedMeadowArea(x, z, x1, z1, x2, z2);

					local fruitType = FruitUtil.FRUITTYPE_GRASS

					local pixelToQm = 2048 / 4096 * 2048 / 4096  / g_currentMission.maxFruitValue; -- 4096px are mapped to 2048m
					local literPerSqm = FruitUtil.fruitIndexToDesc[fruitType].literPerSqm * (1 + 0.5 * (3 - g_currentMission.missionStats.difficulty));
					local qm = area*pixelToQm;

					local deltaLevel = qm*literPerSqm * self.fillScale;
					if deltaLevel > 0 then
						self:setFillLevel(self.fillLevel+deltaLevel, fruitType);
					end;
				end;
			end;

			self.wasToFast = toFast;

			if not self.forageWgnSoundEnabled and self:getIsActiveForSound() then
				playSample(self.forageWgnSound, 0, self.forageWgnSoundVolume, 0);
				setSamplePitch(self.forageWgnSound, self.forageWgnSoundPitchOffset);
				self.forageWgnSoundEnabled = true;
			end;

		end;

		if self.forageWgnSoundEnabled and not self.isTurnedOn then
			stopSample(self.forageWgnSound);
			self.forageWgnSoundEnabled = false;
		end;
		if self.Done.AttacherMoving then
			for k, implement in pairs(self.attacherVehicle.attachedImplements) do
				if implement.object == self then
					local index = implement.jointDescIndex;
					self.attacherVehicleJoint = self.attacherVehicle.attacherJoints[index];
					setJointFrame(self.attacherVehicleJoint.jointIndex,1, self.attacherJoint.node);
				end;
			end;
		end; 
		self:anim("AttacherMoving", false); 

		if self.PU.isDown then
			setJointRotationLimit(self.componentJoints[1].jointIndex, 0, true, -self.PU.limitDown[1], self.PU.limitDown[1]);	
		else
			setJointRotationLimit(self.componentJoints[1].jointIndex, 0, true, -self.PU.limitUp[1], self.PU.limitUp[1]);	
		end;	
	end;
end;

function BergmannRoyal:draw()
	local DrawbarText = (g_i18n:getText("Key_Eng_Deu")..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DRAWBAR_UP).." / "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DRAWBAR_DOWN)..g_i18n:getText("DRAWBAR_UP_DOWN"));
	g_currentMission:addExtraPrintText(DrawbarText);

	if self.isTurnedOn then
		g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
	else
		g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
	end;	
	if not self.PU.isDown then
		g_currentMission:addHelpButtonText(g_i18n:getText("LOWERPU"), InputBinding.IMPLEMENT_EXTRA2);
	else
		g_currentMission:addHelpButtonText(g_i18n:getText("LIFTPU"), InputBinding.IMPLEMENT_EXTRA2);
	end;
    if self.wasToFast then
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getKeyNamesOfDigitalAction(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
    end;
end;

function BergmannRoyal:onDetach()
    if self.deactivateOnDetach then
        BergmannRoyal.onDeactivate(self);
    else
        BergmannRoyal.onDeactivateSounds(self);
    end;
end;

function BergmannRoyal:onLeave()
    if self.deactivateOnLeave then
        BergmannRoyal.onDeactivate(self);
    else
        BergmannRoyal.onDeactivateSounds(self);
    end;
end;

function BergmannRoyal:onDeactivate()
    self.isTurnedOn = false;
    BergmannRoyal.onDeactivateSounds(self);
end;

function BergmannRoyal:onDeactivateSounds()
    if self.forageWgnSoundEnabled then
        stopSample(self.forageWgnSound);
        self.forageWgnSoundEnabled = false;
    end;
end;
